@@ -25,13 +25,13 @@
 #include <QTime>
 #include <QCoreApplication>
 
-#include "fileindexer.h"
+#include "fileextractor.h"
 
 #include "../basicindexingqueue.h"
 #include "../commitqueue.h"
 #include "../database.h"
 #include "../fileindexerconfig.h"
-#include "../lib/filemapping.h"
+#include "filemapping.h"
 
 int main(int argc, char** argv)
 {
@@ -45,11 +45,10 @@ int main(int argc, char** argv)
     QCoreApplication app(argc, argv);
 
     Baloo::BasicIndexingQueue basicIQ(&db, &config);
-    QObject::connect(&basicIQ, SIGNAL(finishedIndexing()), &app, SLOT(quit()));
+    QObject::connect(&basicIQ, &Baloo::BasicIndexingQueue::finishedIndexing, &app, &QCoreApplication::quit);
 
     Baloo::CommitQueue commitQueue(&db);
-    QObject::connect(&basicIQ, SIGNAL(newDocument(uint,Xapian::Document)),
-                     &commitQueue, SLOT(add(uint,Xapian::Document)));
+    QObject::connect(&basicIQ, &Baloo::BasicIndexingQueue::newDocument, &commitQueue, &Baloo::CommitQueue::add);
 
     basicIQ.enqueue(Baloo::FileMapping(QDir::homePath()));
     app.exec();
@@ -68,12 +67,12 @@ int main(int argc, char** argv)
     QHash<QString, int> m_numPerType;
 
     uint totalTime = 0;
-    for (; it != mset.end(); it++) {
+    for (; it != mset.end(); ++it) {
         Baloo::FileMapping fileMap(*it);
         if (!fileMap.fetch(db.sqlDatabase()))
             continue;
 
-        Baloo::FileIndexer* job = new Baloo::FileIndexer(fileMap.id(), fileMap.url());
+        auto job = new Baloo::FileExtractor(fileMap.id(), fileMap.url());
         job->setCustomPath(db.path());
         job->exec();
 
