@@ -36,6 +36,7 @@
 #include <QDBusConnectionInterface>
 
 #include "xapiandatabase.h"
+#include "filestatistics.h"
 
 using namespace Baloo;
 
@@ -47,30 +48,50 @@ void start()
 
 void stop()
 {
-    QDBusMessage message = QDBusMessage::createMethodCall(QLatin1String("org.kde.baloo.file"),
+    QDBusMessage message = QDBusMessage::createMethodCall(QLatin1String("org.kde.baloo"),
                                                           QLatin1String("/indexer"),
-                                                          QLatin1String("org.kde.baloo.file"),
+                                                          QLatin1String("org.kde.baloo"),
                                                           QLatin1String("quit"));
     QDBusConnection::sessionBus().call(message);
 }
 
+void suspend()
+{
+    QDBusMessage message = QDBusMessage::createMethodCall(QLatin1String("org.kde.baloo"),
+                                                          QLatin1String("/indexer"),
+                                                          QLatin1String("org.kde.baloo"),
+                                                          QLatin1String("suspend"));
+    QDBusConnection::sessionBus().call(message);
+}
+
+void resume()
+{
+    QDBusMessage message = QDBusMessage::createMethodCall(QLatin1String("org.kde.baloo"),
+                                                          QLatin1String("/indexer"),
+                                                          QLatin1String("org.kde.baloo"),
+                                                          QLatin1String("resume"));
+    QDBusConnection::sessionBus().call(message);
+}
+
 int main(int argc, char* argv[])
 {
-    KAboutData aboutData("balooctl", i18n("balooctl"), "0.1");
-    aboutData.addAuthor(i18n("Vishesh Handa"), i18n("Maintainer"), "me@vhanda.in");
+    KAboutData aboutData(QLatin1String("balooctl"), i18n("balooctl"), PROJECT_VERSION);
+    aboutData.addAuthor(i18n("Vishesh Handa"), i18n("Maintainer"), QLatin1String("me@vhanda.in"));
 
     KAboutData::setApplicationData(aboutData);
     QCoreApplication app(argc, argv);
 
     QCommandLineParser parser;
-    parser.addPositionalArgument("command", i18n("The command to execute"));
+    parser.addPositionalArgument(QLatin1String("command"), i18n("The command to execute"));
 
-    parser.addPositionalArgument("status", i18n("Print the status of the indexer"));
-    parser.addPositionalArgument("enable", i18n("Enable the file indexer"));
-    parser.addPositionalArgument("disable", i18n("Disable the file indexer"));
-    parser.addPositionalArgument("start", i18n("Start the file indexer"));
-    parser.addPositionalArgument("stop", i18n("Stop the file indexer"));
-    parser.addPositionalArgument("restart", i18n("Restart the file indexer"));
+    parser.addPositionalArgument(QLatin1String("status"), i18n("Print the status of the indexer"));
+    parser.addPositionalArgument(QLatin1String("enable"), i18n("Enable the file indexer"));
+    parser.addPositionalArgument(QLatin1String("disable"), i18n("Disable the file indexer"));
+    parser.addPositionalArgument(QLatin1String("start"), i18n("Start the file indexer"));
+    parser.addPositionalArgument(QLatin1String("stop"), i18n("Stop the file indexer"));
+    parser.addPositionalArgument(QLatin1String("restart"), i18n("Restart the file indexer"));
+    parser.addPositionalArgument(QLatin1String("suspend"), i18n("Suspend the file indexer"));
+    parser.addPositionalArgument(QLatin1String("resume"), i18n("Resume the file indexer"));
 
     parser.process(app);
     if (parser.positionalArguments().isEmpty()) {
@@ -83,7 +104,7 @@ int main(int argc, char* argv[])
     QString command = parser.positionalArguments().first();
     if (command == QLatin1String("status")) {
         QDBusConnection bus = QDBusConnection::sessionBus();
-        bool running = bus.interface()->isServiceRegistered("org.kde.baloo.file");
+        bool running = bus.interface()->isServiceRegistered(QLatin1String("org.kde.baloo"));
 
         if (running) {
             out << "Baloo File Indexer is running\n";
@@ -92,7 +113,7 @@ int main(int argc, char* argv[])
             out << "Baloo File Indexer is NOT running\n";
         }
 
-        const QString path = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + "/baloo/file/";
+        const QString path = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QLatin1String("/baloo/file/");
 
         XapianDatabase database(path);
         Xapian::Database* xdb = database.db();
@@ -113,8 +134,8 @@ int main(int argc, char* argv[])
         int total = xdb->get_doccount();
 
         out << "Indexed " << phaseTwo << " / " << total << " files\n";
-        out << "Failed to index " << failed << " files\n";
         if (failed) {
+            out << "Failed to index " << failed << " files\n";
             out << "File IDs: ";
             Xapian::MSetIterator iter = mset.begin();
             for (; iter != mset.end(); ++iter) {
@@ -139,7 +160,7 @@ int main(int argc, char* argv[])
             isEnabled = false;
         }
 
-        KConfig config("baloofilerc");
+        KConfig config(QLatin1String("baloofilerc"));
         KConfigGroup basicSettings = config.group("Basic Settings");
         basicSettings.writeEntry("Indexing-Enabled", isEnabled);
 
@@ -180,5 +201,25 @@ int main(int argc, char* argv[])
             start();
     }
 
+    if (command == QStringLiteral("suspend")) {
+        suspend();
+        out << "File Indexer suspended\n";
+        return 0;
+    }
+
+    if (command == QStringLiteral("resume")) {
+        resume();
+        out << "File Indexer resumed\n";
+        return 0;
+    }
+
+    if (command == QStringLiteral("fileStatistics")) {
+        const QString path = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QLatin1String("/baloo/file/");
+
+        XapianDatabase database(path);
+        FileStatistics stats(database);
+        stats.compute();
+        stats.print();
+    }
     return 0;
 }
