@@ -72,6 +72,7 @@ void XapianDatabase::replaceDocument(uint id, const XapianDocument& doc)
 
 void XapianDatabase::replaceDocument(uint id, const Xapian::Document& doc)
 {
+    Q_ASSERT(id != 0);
     if (m_writeOnly) {
         try {
             m_wDb.replace_document(id, doc);
@@ -85,6 +86,9 @@ void XapianDatabase::replaceDocument(uint id, const Xapian::Document& doc)
 
 void XapianDatabase::deleteDocument(uint id)
 {
+    if (id == 0)
+        return;
+
     if (m_writeOnly) {
         try {
             m_wDb.delete_document(id);
@@ -96,6 +100,11 @@ void XapianDatabase::deleteDocument(uint id)
     m_docsToRemove << id;
 }
 
+bool XapianDatabase::haveChanges() const
+{
+    return !m_docsToAdd.isEmpty() || !m_docsToRemove.isEmpty();
+}
+
 void XapianDatabase::commit()
 {
     if (m_writeOnly) {
@@ -108,7 +117,7 @@ void XapianDatabase::commit()
         return;
     }
 
-    if (m_docsToAdd.isEmpty() && m_docsToRemove.isEmpty()) {
+    if (!haveChanges()) {
         return;
     }
 
@@ -119,7 +128,8 @@ void XapianDatabase::commit()
         try {
             wdb.replace_document(doc.first, doc.second);
         }
-        catch (const Xapian::Error&) {
+        catch (const Xapian::Error& err) {
+            qDebug() << "ERROR" << err.get_description().c_str();
         }
     }
 
@@ -128,7 +138,8 @@ void XapianDatabase::commit()
         try {
             wdb.delete_document(id);
         }
-        catch (const Xapian::Error&) {
+        catch (const Xapian::Error& err) {
+            qDebug() << "ERROR" << err.get_description().c_str();
         }
     }
 
@@ -173,7 +184,7 @@ Xapian::WritableDatabase XapianDatabase::createWritableDb()
 {
     // We need to keep sleeping for a required amount, until we reach
     // a threshold. That's when we decide to abort?
-    for (int i = 1; i <= 20; i++) {
+    for (int i = 1; i <= 20; ++i) {
         try {
             Xapian::WritableDatabase wdb(m_path, Xapian::DB_CREATE_OR_OPEN);
             return wdb;
